@@ -1,17 +1,36 @@
+/*
+Copyright (c) 2003, The Regents of the University of California, through
+Lawrence Berkeley National Laboratory (subject to receipt of any required 
+approvals from U.S. Dept. of Energy) 
+
+All rights reserved. 
+
+The source code is distributed under BSD license, see the file License.txt
+at the top-level directory.
+*/
 
 /*
- * -- SuperLU routine (version 3.0) --
+ * -- SuperLU routine (version 7.0.0) --
  * Univ. of California Berkeley, Xerox Palo Alto Research Center,
  * and Lawrence Berkeley National Lab.
  * October 15, 2003
+ * August 2024
  *
  */
-/*
- * File name:		sdrive.c
- * Purpose:             MAIN test program
+
+/*! \file
+ * SDRIVE is the main test program for the FLOAT linear 
+ * equation driver routines SGSSV and SGSSVX.
+ * 
+ * The program is invoked by a shell script file -- stest.csh.
+ * The output from the tests are written into a file -- stest.out.
+ *
+ * \ingroup TestingS
  */
+#include <getopt.h>
 #include <string.h>
-#include "ssp_defs.h"
+#include "slu_sdefs.h"
+#include "MATGEN/matgen.h"
 
 #define NTESTS    5      /* Number of test types */
 #define NTYPES    11     /* Number of matrix types */
@@ -21,26 +40,22 @@
 #define	FMT2      "%10s:fact=%4d, trans=%4d, equed=%c, n=%d, imat=%d, test(%d)=%12.5g\n"
 #define FMT3      "%10s:info=%d, izero=%d, n=%d, nrhs=%d, imat=%d, nfail=%d\n"
 
+static void
+parse_command_line(int argc, char *argv[], char *matrix_type,
+		   int *n, int *w, int *relax, int *nrhs, int *maxsuper,
+		   int *rowblk, int *colblk, int_t *lwork, double *u, FILE **fp);
 
-main(int argc, char *argv[])
-{
-/* 
- * Purpose
- * =======
- *
- * SDRIVE is the main test program for the FLOAT linear 
- * equation driver routines SGSSV and SGSSVX.
- * 
- * The program is invoked by a shell script file -- stest.csh.
- * The output from the tests are written into a file -- stest.out.
- *
- * =====================================================================
+/*!
+ * Entry point of test program.
  */
+int main(int argc, char *argv[])
+{
     float         *a, *a_save;
-    int            *asub, *asub_save;
-    int            *xa, *xa_save;
+    int_t          *asub, *asub_save;
+    int_t          *xa, *xa_save;
     SuperMatrix  A, B, X, L, U;
     SuperMatrix  ASAV, AC;
+    GlobalLU_t   Glu; /* Not needed on return. */
     mem_usage_t    mem_usage;
     int            *perm_r; /* row permutation from partial pivoting */
     int            *perm_c, *pc_save; /* column permutation */
@@ -50,9 +65,10 @@ main(int argc, char *argv[])
     float         *ferr, *berr;
     float         *rwork;
     float	   *wwork;
-    void           *work;
-    int            info, lwork, nrhs, panel_size, relax;
-    int            m, n, nnz;
+    void           *work = NULL;
+    int            nrhs, panel_size, relax;
+    int            m, n, info1;
+    int_t          nnz, lwork, info;
     float         *xact;
     float         *rhsb, *solx, *bsav;
     int            ldb, ldx;
@@ -60,12 +76,14 @@ main(int argc, char *argv[])
     int            i, j, k1;
     float         rowcnd, colcnd, amax;
     int            maxsuper, rowblk, colblk;
-    int            prefact, nofact, equil, iequed;
+    int            prefact, equil, iequed;
     int            nt, nrun, nfail, nerrs, imat, fimat, nimat;
     int            nfact, ifact, itran;
-    int            kl, ku, mode, lda;
-    int            zerot, izero, ioff;
-    float         anorm, cndnum, u, drop_tol = 0.;
+    int            kl, ku, mode, lda, ioff;
+    int            zerot; /* indicate whether the matrix is singular */
+    int            izero; /* indicate the first column that is entirely zero */
+    double         u;
+    float         anorm, cndnum;
     float         *Afull;
     float         result[NTESTS];
     superlu_options_t options;
@@ -73,7 +91,8 @@ main(int argc, char *argv[])
     trans_t        trans;
     SuperLUStat_t  stat;
     static char    matrix_type[8];
-    static char    equed[1], path[3], sym[1], dist[1];
+    static char    equed[1], path[4], sym[1], dist[1];
+    FILE           *fp;
 
     /* Fixed set of parameters */
     int            iseed[]  = {1988, 1989, 1990, 1991};
@@ -83,23 +102,22 @@ main(int argc, char *argv[])
     static trans_t transs[]  = {NOTRANS, TRANS, CONJ};
 
     /* Some function prototypes */ 
-    static void parse_command_line();
-    extern int sp_sget01(int, int, SuperMatrix *, SuperMatrix *, 
-		         SuperMatrix *, int *, float *);
-    extern int sp_sget02(trans_t, int, int, int, SuperMatrix *, float *,
-                         int, float *, int, float *resid);
-    extern int sp_sget04(int, int, float *, int, 
-                         float *, int, float rcond, float *resid);
-    extern int sp_sget07(trans_t, int, int, SuperMatrix *, float *, int,
+    extern int sgst01(int, int, SuperMatrix *, SuperMatrix *, 
+		      SuperMatrix *, int *, int *, float *);
+    extern int sgst02(trans_t, int, int, int, SuperMatrix *, float *,
+                      int, float *, int, float *resid);
+    extern int sgst04(int, int, float *, int, 
+                      float *, int, float rcond, float *resid);
+    extern int sgst07(trans_t, int, int, SuperMatrix *, float *, int,
                          float *, int, float *, int, 
                          float *, float *, float *);
-    extern int slatb4_(char *, int *, int *, int *, char *, int *, int *, 
+    extern int slatb4_slu(char *, int *, int *, int *, char *, int *, int *, 
 	               float *, int *, float *, char *);
-    extern int slatms_(int *, int *, char *, int *, char *, float *d,
+    extern int slatms_slu(int *, int *, char *, int *, char *, float *d,
                        int *, float *, float *, int *, int *,
                        char *, float *, int *, float *, int *);
     extern int sp_sconvert(int, int, float *, int, int, int,
-	                   float *a, int *, int *, int *);
+	                   float *a, int_t *, int_t *, int_t *);
 
 
     /* Executable statements */
@@ -119,11 +137,11 @@ main(int argc, char *argv[])
     strcpy(matrix_type, "LA");
     parse_command_line(argc, argv, matrix_type, &n,
 		       &panel_size, &relax, &nrhs, &maxsuper,
-		       &rowblk, &colblk, &lwork, &u);
+		       &rowblk, &colblk, &lwork, &u, &fp);
     if ( lwork > 0 ) {
 	work = SUPERLU_MALLOC(lwork);
 	if ( !work ) {
-	    fprintf(stderr, "expert: cannot allocate %d bytes\n", lwork);
+	    fprintf(stderr, "expert: cannot allocate %lld bytes\n", (long long) lwork);
 	    exit (-1);
 	}
     }
@@ -134,7 +152,7 @@ main(int argc, char *argv[])
     options.PrintStat = NO;
     options.PivotGrowth = YES;
     options.ConditionNumber = YES;
-    options.IterRefine = SINGLE;
+    options.IterRefine = SLU_SINGLE;
     
     if ( strcmp(matrix_type, "LA") == 0 ) {
 	/* Test LAPACK matrix suite. */
@@ -148,22 +166,24 @@ main(int argc, char *argv[])
     } else {
 	/* Read a sparse matrix */
 	fimat = nimat = 0;
-	sreadhb(&m, &n, &nnz, &a, &asub, &xa);
+	sreadhb(fp, &m, &n, &nnz, &a, &asub, &xa);
     }
 
     sallocateA(n, nnz, &a_save, &asub_save, &xa_save);
     rhsb = floatMalloc(m * nrhs);
     bsav = floatMalloc(m * nrhs);
     solx = floatMalloc(n * nrhs);
+    xact = floatMalloc(n * nrhs);
+    wwork = floatCalloc( SUPERLU_MAX(m,n) * SUPERLU_MAX(4,nrhs) );
+
     ldb  = m;
     ldx  = n;
     sCreate_Dense_Matrix(&B, m, nrhs, rhsb, ldb, SLU_DN, SLU_S, SLU_GE);
     sCreate_Dense_Matrix(&X, n, nrhs, solx, ldx, SLU_DN, SLU_S, SLU_GE);
-    xact = floatMalloc(n * nrhs);
-    etree   = intMalloc(n);
-    perm_r  = intMalloc(n);
-    perm_c  = intMalloc(n);
-    pc_save = intMalloc(n);
+    etree   = int32Malloc(n);
+    perm_r  = int32Malloc(n);
+    perm_c  = int32Malloc(n);
+    pc_save = int32Malloc(n);
     R       = (float *) SUPERLU_MALLOC(m*sizeof(float));
     C       = (float *) SUPERLU_MALLOC(n*sizeof(float));
     ferr    = (float *) SUPERLU_MALLOC(nrhs*sizeof(float));
@@ -176,7 +196,6 @@ main(int argc, char *argv[])
     if ( !ferr ) ABORT("SUPERLU_MALLOC fails for ferr");
     if ( !berr ) ABORT("SUPERLU_MALLOC fails for berr");
     if ( !rwork ) ABORT("SUPERLU_MALLOC fails for rwork");
-    wwork   = floatCalloc( SUPERLU_MAX(m,n) * SUPERLU_MAX(4,nrhs) );
 
     for (i = 0; i < n; ++i) perm_c[i] = pc_save[i] = i;
     options.ColPerm = MY_PERMC;
@@ -192,15 +211,15 @@ main(int argc, char *argv[])
 	    
 	    /* Set up parameters with SLATB4 and generate a test matrix
 	       with SLATMS.  */
-	    slatb4_(path, &imat, &n, &n, sym, &kl, &ku, &anorm, &mode,
+	    slatb4_slu(path, &imat, &n, &n, sym, &kl, &ku, &anorm, &mode,
 		    &cndnum, dist);
 
-	    slatms_(&n, &n, dist, iseed, sym, &rwork[0], &mode, &cndnum,
+	    slatms_slu(&n, &n, dist, iseed, sym, &rwork[0], &mode, &cndnum,
 		    &anorm, &kl, &ku, "No packing", Afull, &lda,
-		    &wwork[0], &info);
+		    &wwork[0], &info1);
 
-	    if ( info ) {
-		printf(FMT3, "SLATMS", info, izero, n, nrhs, imat, nfail);
+	    if ( info1 ) {
+		printf(FMT3, "SLATMS", info1, izero, n, nrhs, imat, nfail);
 		continue;
 	    }
 
@@ -219,14 +238,14 @@ main(int argc, char *argv[])
 			    Afull[ioff + i + j*lda] = zero;
 		}
 	    } else {
-		izero = 0;
+		izero = n+1; /* none of the column is zero */
 	    }
 
 	    /* Convert to sparse representation. */
 	    sp_sconvert(n, n, Afull, lda, kl, ku, a, asub, xa, &nnz);
 
 	} else {
-	    izero = 0;
+	    izero = n+1; /* none of the column is zero */
 	    zerot = 0;
 	}
 	
@@ -256,7 +275,6 @@ main(int argc, char *argv[])
 		    prefact   = ( options.Fact == FACTORED ||
 				  options.Fact == SamePattern_SameRowPerm );
                                 /* Need a first factor */
-		    nofact    = (options.Fact != FACTORED);  /* Not factored */
 
 		    /* Restore the matrix A. */
 		    sCopy_CompCol_Matrix(&ASAV, &A);
@@ -267,17 +285,17 @@ main(int argc, char *argv[])
                         if ( equil || iequed ) {
 			    /* Compute row and column scale factors to
 			       equilibrate matrix A.    */
-			    sgsequ(&A, R, C, &rowcnd, &colcnd, &amax, &info);
+			    sgsequ(&A, R, C, &rowcnd, &colcnd, &amax, &info1);
 
 			    /* Force equilibration. */
 			    if ( !info && n > 0 ) {
-				if ( lsame_(equed, "R") ) {
+				if ( strncmp(equed, "R", 1)==0 ) {
 				    rowcnd = 0.;
 				    colcnd = 1.;
-				} else if ( lsame_(equed, "C") ) {
+				} else if ( strncmp(equed, "C", 1)==0 ) {
 				    rowcnd = 1.;
 				    colcnd = 0.;
-				} else if ( lsame_(equed, "B") ) {
+				} else if ( strncmp(equed, "B", 1)==0 ) {
 				    rowcnd = 0.;
 				    colcnd = 0.;
 				}
@@ -298,16 +316,16 @@ main(int argc, char *argv[])
 			sp_preorder(&options, &A, perm_c, etree, &AC);
 
 			/* Factor the matrix AC. */
-			sgstrf(&options, &AC, drop_tol, relax, panel_size,
+			sgstrf(&options, &AC, relax, panel_size,
                                etree, work, lwork, perm_c, perm_r, &L, &U,
-                               &stat, &info);
+                               &Glu, &stat, &info);
 
 			if ( info ) { 
-                            printf("** First factor: info %d, equed %c\n",
-				   info, *equed);
+                            printf("** First factor: info %lld, equed %c\n",
+				   (long long) info, *equed);
                             if ( lwork == -1 ) {
-                                printf("** Estimated memory: %d bytes\n",
-                                        info - n);
+                                printf("** Estimated memory: %lld bytes\n",
+                                        (long long) info - n);
                                 exit(0);
                             }
                         }
@@ -341,18 +359,20 @@ main(int argc, char *argv[])
 			    
 			    if ( info && info != izero ) {
                                 printf(FMT3, "sgssv",
-				       info, izero, n, nrhs, imat, nfail);
+				       (int) info, izero, n, nrhs, imat, nfail);
 			    } else {
-                                /* Reconstruct matrix from factors and
-	                           compute residual. */
-                                sp_sget01(m, n, &A, &L, &U, perm_r, &result[0]);
+                                /* Reconstruct matrix from factors and compute residual.
+				 * Only compute the leading 'izero' nonzero columns.
+				 */
+                                sgst01(m, izero-1, &A, &L, &U, perm_c, perm_r,
+                                         &result[0]);
 				nt = 1;
-				if ( izero == 0 ) {
+				if ( izero == (n+1) ) {
 				    /* Compute residual of the computed
 				       solution. */
 				    sCopy_Dense_Matrix(m, nrhs, rhsb, ldb,
 						       wwork, ldb);
-				    sp_sget02(trans, m, n, nrhs, &A, solx,
+				    sgst02(trans, m, n, nrhs, &A, solx,
                                               ldx, wwork,ldb, &result[1]);
 				    nt = 2;
 				}
@@ -393,11 +413,12 @@ main(int argc, char *argv[])
 			   and error bounds using sgssvx.      */
 			sgssvx(&options, &A, perm_c, perm_r, etree,
                                equed, R, C, &L, &U, work, lwork, &B, &X, &rpg,
-                               &rcond, ferr, berr, &mem_usage, &stat, &info);
+                               &rcond, ferr, berr, &Glu,
+			       &mem_usage, &stat, &info);
 
 			if ( info && info != izero ) {
 			    printf(FMT3, "sgssvx",
-				   info, izero, n, nrhs, imat, nfail);
+				   (int) info, izero, n, nrhs, imat, nfail);
                             if ( lwork == -1 ) {
                                 printf("** Estimated memory: %.0f bytes\n",
                                         mem_usage.total_needed);
@@ -405,9 +426,11 @@ main(int argc, char *argv[])
                             }
 			} else {
 			    if ( !prefact ) {
-			    	/* Reconstruct matrix from factors and
-	 			   compute residual. */
-                                sp_sget01(m, n, &A, &L, &U, perm_r, &result[0]);
+			    	/* Reconstruct matrix from factors and compute residual.
+				 * Only compute the leading 'izero' nonzero columns.
+				 */
+                                sgst01(m, izero-1, &A, &L, &U, perm_c, perm_r,
+                                         &result[0]);
 				k1 = 0;
 			    } else {
 			   	k1 = 1;
@@ -417,17 +440,17 @@ main(int argc, char *argv[])
 				/* Compute residual of the computed solution.*/
 				sCopy_Dense_Matrix(m, nrhs, bsav, ldb,
 						  wwork, ldb);
-				sp_sget02(trans, m, n, nrhs, &ASAV, solx, ldx,
+				sgst02(trans, m, n, nrhs, &ASAV, solx, ldx,
 					  wwork, ldb, &result[1]);
 
 				/* Check solution from generated exact
 				   solution. */
-				sp_sget04(n, nrhs, solx, ldx, xact, ldx, rcond,
+				sgst04(n, nrhs, solx, ldx, xact, ldx, rcond,
 					  &result[2]);
 
 				/* Check the error bounds from iterative
 				   refinement. */
-				sp_sget07(trans, n, nrhs, &ASAV, bsav, ldb,
+				sgst07(trans, n, nrhs, &ASAV, bsav, ldb,
 					  solx, ldx, xact, ldx, ferr, berr,
 					  &result[3]);
 
@@ -459,13 +482,17 @@ main(int argc, char *argv[])
     if ( !info ) {
 	PrintPerf(&L, &U, &mem_usage, rpg, rcond, ferr, berr, equed);
     }
-#endif    
+#endif
+        Destroy_SuperMatrix_Store(&A);
+        Destroy_SuperMatrix_Store(&ASAV);
+        StatFree(&stat);
 
     } /* for imat ... */
 
     /* Print a summary of the results. */
     PrintSumm("SGE", nfail, nrun, nerrs);
 
+    if ( strcmp(matrix_type, "LA") == 0 ) SUPERLU_FREE (Afull);
     SUPERLU_FREE (rhsb);
     SUPERLU_FREE (bsav);
     SUPERLU_FREE (solx);    
@@ -482,30 +509,34 @@ main(int argc, char *argv[])
     SUPERLU_FREE (wwork);
     Destroy_SuperMatrix_Store(&B);
     Destroy_SuperMatrix_Store(&X);
+#if 0
     Destroy_CompCol_Matrix(&A);
     Destroy_CompCol_Matrix(&ASAV);
+#else
+    SUPERLU_FREE(a); SUPERLU_FREE(asub); SUPERLU_FREE(xa);
+    SUPERLU_FREE(a_save); SUPERLU_FREE(asub_save); SUPERLU_FREE(xa_save);
+#endif
     if ( lwork > 0 ) {
 	SUPERLU_FREE (work);
 	Destroy_SuperMatrix_Store(&L);
 	Destroy_SuperMatrix_Store(&U);
     }
-    StatFree(&stat);
 
-    return 0;
-}
+    return (nfail == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+} /* end main */
 
-/*  
+/*!
  * Parse command line options to get relaxed snode size, panel size, etc.
  */
 static void
 parse_command_line(int argc, char *argv[], char *matrix_type,
 		   int *n, int *w, int *relax, int *nrhs, int *maxsuper,
-		   int *rowblk, int *colblk, int *lwork, double *u)
+		   int *rowblk, int *colblk, int_t *lwork, double *u, FILE **fp)
 {
     int c;
     extern char *optarg;
 
-    while ( (c = getopt(argc, argv, "ht:n:w:r:s:m:b:c:l:")) != EOF ) {
+    while ( (c = getopt(argc, argv, "ht:n:w:r:s:m:b:c:l:u:f:")) != EOF ) {
 	switch (c) {
 	  case 'h':
 	    printf("Options:\n");
@@ -533,6 +564,12 @@ parse_command_line(int argc, char *argv[], char *matrix_type,
 	            break;
 	  case 'u': *u = atof(optarg); 
 	            break;
+          case 'f':
+                    if ( !(*fp = fopen(optarg, "r")) ) {
+                        ABORT("File does not exist");
+                    }
+                    printf(".. test sparse matrix in file: %s\n", optarg);
+                    break;
   	}
     }
 }
